@@ -18,44 +18,52 @@
  */
 
 #include "KSMainWindow.h"
+
 #include "Config.h"
+#include "SettingsDialog/SettingsDialog.h"
+
+#include <KAboutData>
+#include <KGuiItem>
+#include <KHelpMenu>
+#include <KIO/OpenFileManagerWindowJob>
+#include <KLocalizedString>
+#include <KNS3/KMoreToolsMenuFactory>
+#include <KStandardAction>
+#include <KStandardGuiItem>
+#include <KWindowSystem>
 
 #include <QDesktopServices>
 #include <QJsonArray>
 #include <QPrintDialog>
-#include <QShortcut>
-#include <QTimer>
 #include <QPushButton>
+#include <QTimer>
 #include <QVBoxLayout>
-
 #ifdef XCB_FOUND
 #include <QX11Info>
 #include <xcb/xcb.h>
 #endif
 
-#include <KLocalizedString>
-#include <KGuiItem>
-#include <KStandardAction>
-#include <KStandardGuiItem>
-#include <KHelpMenu>
-#include <KAboutData>
-#include <KWindowSystem>
 
-#include "SettingsDialog/SettingsDialog.h"
-#include "ExportMenu.h"
-#include "ExportManager.h"
-#include "SpectacleConfig.h"
+static const int DEFAULT_WINDOW_HEIGHT = 420;
+static const int DEFAULT_WINDOW_WIDTH = 840;
+static const int MAXIMUM_WINDOW_WIDTH = 1000;
 
 KSMainWindow::KSMainWindow(bool onClickAvailable, QWidget *parent) :
     QDialog(parent),
     mKSWidget(new KSWidget),
     mDivider(new QFrame),
     mDialogButtonBox(new QDialogButtonBox),
+    mConfigureButton(new QToolButton),
+    mToolsButton(new QPushButton),
     mSendToButton(new QPushButton),
     mClipboardButton(new QToolButton),
     mSaveButton(new QToolButton),
     mSaveMenu(new QMenu),
+    mSaveAsAction(new QAction(this)),
+    mSaveAction(new QAction(this)),
     mMessageWidget(new KMessageWidget),
+    mToolsMenu(new QMenu),
+    mScreenRecorderToolsMenu(new QMenu),
     mExportMenu(new ExportMenu(this)),
     mOnClickAvailable(onClickAvailable)
 {
@@ -123,34 +131,58 @@ void KSMainWindow::init()
 
     // the Button Bar
 
-    mDialogButtonBox->setStandardButtons(QDialogButtonBox::Help | QDialogButtonBox::Discard);
+    mDialogButtonBox->setStandardButtons(QDialogButtonBox::Help);
 
-    KGuiItem::assign(mSendToButton, KGuiItem(i18n("Export Image...")));
-    mSendToButton->setIcon(QIcon::fromTheme(QStringLiteral("application-x-executable")));
+    mConfigureButton->setDefaultAction(KStandardAction::preferences(this, SLOT(showPreferencesDialog()), this));
+    mConfigureButton->setText(i18n("Configure..."));
+    mConfigureButton->setToolTip(i18n("Change Spectacle's settings."));
+    mConfigureButton->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
+    mDialogButtonBox->addButton(mConfigureButton, QDialogButtonBox::ResetRole);
+
+    KGuiItem::assign(mToolsButton, KGuiItem(i18n("Tools")));
+    mToolsButton->setIcon(QIcon::fromTheme(QStringLiteral("application-menu")));
+    mDialogButtonBox->addButton(mToolsButton, QDialogButtonBox::ActionRole);
+    mToolsButton->setMenu(mToolsMenu);
+
+    KGuiItem::assign(mSendToButton, KGuiItem(i18n("Export")));
+    mSendToButton->setIcon(QIcon::fromTheme(QStringLiteral("document-share")));
     mDialogButtonBox->addButton(mSendToButton, QDialogButtonBox::ActionRole);
 
     mClipboardButton->setDefaultAction(KStandardAction::copy(this, SLOT(sendToClipboard()), this));
     mClipboardButton->setText(i18n("Copy To Clipboard"));
     mClipboardButton->setToolTip(i18n("Copy the current screenshot image to the clipboard."));
-    mClipboardButton->setIcon(QIcon::fromTheme(QStringLiteral("edit-copy")));
     mClipboardButton->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
     mDialogButtonBox->addButton(mClipboardButton, QDialogButtonBox::ActionRole);
 
     mSaveButton->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
     mSaveButton->setMenu(mSaveMenu);
     mSaveButton->setPopupMode(QToolButton::MenuButtonPopup);
-    buildSaveMenu();
     mDialogButtonBox->addButton(mSaveButton, QDialogButtonBox::ActionRole);
 
-    QShortcut *shortcut = new QShortcut(QKeySequence(Qt::Key_Escape), mDialogButtonBox->button(QDialogButtonBox::Discard));
-    connect(shortcut, &QShortcut::activated, qApp, &QApplication::quit);
-    connect(mDialogButtonBox->button(QDialogButtonBox::Discard), &QPushButton::clicked, qApp, &QApplication::quit);
-
     // the help menu
-
     KHelpMenu *helpMenu = new KHelpMenu(this, KAboutData::applicationData(), true);
     mDialogButtonBox->button(QDialogButtonBox::Help)->setMenu(helpMenu->menu());
 
+    // the tools menu
+    mToolsMenu->addAction(QIcon::fromTheme(QStringLiteral("document-open-folder")),
+                          i18n("Open Screenshots Folder"),
+                          this, &KSMainWindow::openScreenshotsFolder);
+    mToolsMenu->addAction(KStandardAction::print(this, &KSMainWindow::showPrintDialog, this));
+    mScreenRecorderToolsMenu = mToolsMenu->addMenu(i18n("Record Screen"));
+    connect(mScreenRecorderToolsMenu, &QMenu::aboutToShow, [this]()
+    {
+        mScreenrecorderToolsMenuFactory.reset(new KMoreToolsMenuFactory(QStringLiteral("spectacle/screenrecorder-tools")));
+        mScreenRecorderToolsMenu->clear();
+        mScreenrecorderToolsMenuFactory->fillMenuFromGroupingNames(mScreenRecorderToolsMenu, { QStringLiteral("screenrecorder") });
+    } );
+
+    // the save menu
+    mSaveAsAction = KStandardAction::saveAs(this, &KSMainWindow::saveAs, this);
+    mSaveAction = KStandardAction::save(this, &KSMainWindow::save, this);
+    mSaveMenu->addAction(mSaveAsAction);
+    mSaveMenu->addAction(mSaveAction);
+    setDefaultSaveAction();
+
     // message widget
     connect(mMessageWidget, &KMessageWidget::linkActivated, this, [](const QString &str) { QDesktopServices::openUrl(QUrl(str)); } );
 
@@ -176,54 +208,42 @@ void KSMainWindow::init()
     if (!mOnClickAvailable) {
         mKSWidget->disableOnClick();
     }
-    resize(QSize(840, 420).expandedTo(minimumSize()));
+    resize(QSize(DEFAULT_WINDOW_WIDTH, DEFAULT_WINDOW_HEIGHT).expandedTo(minimumSize()));
 
+    // Allow Ctrl+Q to quit the app
+    QAction *actionQuit = KStandardAction::quit(qApp, &QApplication::quit, this);
+    actionQuit->setShortcut(QKeySequence::Quit);
+    addAction(actionQuit);
 
     // done with the init
 }
 
-void KSMainWindow::buildSaveMenu()
+int KSMainWindow::windowWidth(const QPixmap &pixmap) const
 {
-    // first clear the menu
-    mSaveMenu->clear();
+    // Calculates what the width of the window should be for the captured image to perfectly fit
+    // the area reserved for the image, with the height already set.
 
-    // get our actions in order
-    QAction *actionSave = KStandardAction::save(this, SLOT(save()), this);
-    QAction *actionSaveAs = KStandardAction::saveAs(this, SLOT(saveAs()), this);
-    QAction *actionSaveExit = new QAction(QIcon::fromTheme(QStringLiteral("document-save")), i18n("Save &&& Exit"), this);
-    actionSaveExit->setToolTip(i18n("Save screenshot in your Pictures directory and exit"));
-    actionSaveExit->setShortcut(QKeySequence(QKeySequence::Quit));
-    connect(actionSaveExit, &QAction::triggered, this, &KSMainWindow::saveAndExit);
+    const float pixmapAspectRatio = (float)pixmap.width() / pixmap.height();
+    const int imageHeight = mKSWidget->height() - 2 * layout()->spacing();
+    const int imageWidth = pixmapAspectRatio * imageHeight;
 
-    // static or dynamic
-    SpectacleConfig *cfgManager = SpectacleConfig::instance();
-    int switchState = cfgManager->useDynamicSaveButton() ? cfgManager->lastUsedSaveMode() : 0;
+    int alignedWindowWidth = qMin(mKSWidget->imagePaddingWidth() + imageWidth, MAXIMUM_WINDOW_WIDTH);
+    alignedWindowWidth += layout()->contentsMargins().left() + layout()->contentsMargins().right();
+    alignedWindowWidth += 2; // margins is removing 1 - 1 pixel for some reason
 
-    // put the actions in order
-    switch (switchState) {
-    case 1:
-        mSaveButton->setDefaultAction(actionSave);
-        mSaveMenu->addAction(actionSaveExit);
-        mSaveMenu->addAction(actionSaveAs);
+    return alignedWindowWidth;
+}
+
+void KSMainWindow::setDefaultSaveAction()
+{
+    switch (SpectacleConfig::instance()->lastUsedSaveMode()) {
+    case SaveMode::SaveAs:
+        mSaveButton->setDefaultAction(mSaveAsAction);
         break;
-    case 2:
-        mSaveButton->setDefaultAction(actionSaveAs);
-        mSaveMenu->addAction(actionSaveExit);
-        mSaveMenu->addAction(actionSave);
-        break;
-    case 0:
-    default:
-        mSaveButton->setDefaultAction(actionSaveExit);
-        mSaveMenu->addAction(actionSave);
-        mSaveMenu->addAction(actionSaveAs);
+    case SaveMode::Save:
+        mSaveButton->setDefaultAction(mSaveAction);
         break;
     }
-
-    // finish off building the menu
-    mSaveMenu->addAction(KStandardAction::print(this, SLOT(showPrintDialog()), this));
-    mSaveMenu->addSeparator();
-    mSaveMenu->addAction(QIcon::fromTheme(QStringLiteral("applications-system")), i18n("Preferences"),
-                         this, SLOT(showPreferencesDialog()));
 }
 
 // overrides
@@ -255,8 +275,9 @@ void KSMainWindow::setScreenshotAndShow(const QPixmap &pixmap)
     setWindowTitle(i18nc("Unsaved Screenshot", "Unsaved[*]"));
     setWindowModified(true);
 
-    KGuiItem::assign(mDialogButtonBox->button(QDialogButtonBox::Discard), KStandardGuiItem::discard());
     show();
+
+    resize(QSize(windowWidth(pixmap), DEFAULT_WINDOW_HEIGHT));
 }
 
 void KSMainWindow::showPrintDialog()
@@ -270,6 +291,27 @@ void KSMainWindow::showPrintDialog()
     delete printer;
 }
 
+void KSMainWindow::openScreenshotsFolder()
+{
+    // Highlight last screenshot in file manager if user saved at least once,
+    // or open default directory as determined by save button
+    QUrl location = ExportManager::instance()->lastSavePath();
+    if (!ExportManager::instance()->isFileExists(location)) {
+        switch(SpectacleConfig::instance()->lastUsedSaveMode()) {
+        case SaveMode::Save:
+            location = QUrl::fromLocalFile(ExportManager::instance()->saveLocation() + QStringLiteral("/"));
+            break;
+        case SaveMode::SaveAs:
+            location = SpectacleConfig::instance()->lastSaveAsLocation();  // already has a "/" at the end
+            break;
+        }
+        if (!ExportManager::instance()->isFileExists(location)) {
+            location = QUrl(QStandardPaths::writableLocation(QStandardPaths::PicturesLocation) + QStringLiteral("/"));
+        }
+    }
+    KIO::highlightInFileManager({location});
+}
+
 void KSMainWindow::showImageSharedFeedback(bool error, const QString &message)
 {
     if (error) {
@@ -293,6 +335,12 @@ void KSMainWindow::sendToClipboard()
 {
     ExportManager::instance()->doCopyToClipboard();
 
+    if (SpectacleConfig::instance()->quitAfterSaveOrCopyChecked()) {
+        qApp->setQuitOnLastWindowClosed(false);
+        hide();
+        QTimer::singleShot(250, qApp, &QApplication::quit);
+    }
+
     mMessageWidget->setMessageType(KMessageWidget::Information);
     mMessageWidget->setText(i18n("The screenshot has been copied to the clipboard."));
     mMessageWidget->setIcon(QIcon::fromTheme(QStringLiteral("dialog-information")));
@@ -311,27 +359,35 @@ void KSMainWindow::setScreenshotWindowTitle(QUrl location)
 {
     setWindowTitle(location.fileName());
     setWindowModified(false);
-    KGuiItem::assign(mDialogButtonBox->button(QDialogButtonBox::Discard), KStandardGuiItem::quit());
 }
 
 void KSMainWindow::save()
 {
-    SpectacleConfig::instance()->setLastUsedSaveMode(1);
-    buildSaveMenu();
-    ExportManager::instance()->doSave();
+    SpectacleConfig::instance()->setLastUsedSaveMode(SaveMode::Save);
+    setDefaultSaveAction();
+
+    if (SpectacleConfig::instance()->quitAfterSaveOrCopyChecked()) {
+        ExportManager::instance()->doSave(QUrl(), true);
+        qApp->setQuitOnLastWindowClosed(false);
+        hide();
+    }
+    else {
+        ExportManager::instance()->doSave();
+    }
 }
 
 void KSMainWindow::saveAs()
 {
-    SpectacleConfig::instance()->setLastUsedSaveMode(2);
-    buildSaveMenu();
-    ExportManager::instance()->doSaveAs(this);
-}
+    SpectacleConfig::instance()->setLastUsedSaveMode(SaveMode::SaveAs);
+    setDefaultSaveAction();
 
-void KSMainWindow::saveAndExit()
-{
-    SpectacleConfig::instance()->setLastUsedSaveMode(0);
-    qApp->setQuitOnLastWindowClosed(false);
-    ExportManager::instance()->doSave(QUrl(), true);
-    hide();
+    if (SpectacleConfig::instance()->quitAfterSaveOrCopyChecked()) {
+        if (ExportManager::instance()->doSaveAs(this, true)) {
+            qApp->setQuitOnLastWindowClosed(false);
+            hide();
+        }
+    }
+    else {
+        ExportManager::instance()->doSaveAs(this, false);
+    }
 }
