@@ -19,23 +19,21 @@
 
 #include "KSWidget.h"
 
-#include <QGridLayout>
-#include <QHBoxLayout>
-#include <QVBoxLayout>
-#include <QFormLayout>
-#include <QComboBox>
+#include "KSImageWidget.h"
+#include "SmartSpinBox.h"
+#include "SpectacleConfig.h"
+
+#include <KLocalizedString>
+
 #include <QCheckBox>
+#include <QComboBox>
+#include <QFormLayout>
+#include <QGridLayout>
 #include <QLabel>
 #include <QPushButton>
 #include <QShortcut>
 #include <QTimer>
 
-#include <KLocalizedString>
-
-#include "KSImageWidget.h"
-#include "SmartSpinBox.h"
-
-#include "SpectacleConfig.h"
 
 KSWidget::KSWidget(QWidget *parent) :
     QWidget(parent)
@@ -52,8 +50,7 @@ KSWidget::KSWidget(QWidget *parent) :
 
     // the capture mode options first
 
-    mCaptureModeLabel = new QLabel(this);
-    mCaptureModeLabel->setText(i18n("<b>Capture Mode</b>"));
+    mCaptureModeLabel = new QLabel(i18n("<b>Capture Mode</b>"), this);
 
     mCaptureArea = new QComboBox(this);
     mCaptureArea->insertItem(1, i18n("Full Screen (All Monitors)"), ImageGrabber::FullScreen);
@@ -74,11 +71,10 @@ KSWidget::KSWidget(QWidget *parent) :
     connect(mDelayMsec, static_cast<void (SmartSpinBox::*)(qreal)>(&SmartSpinBox::valueChanged),
             configManager, &SpectacleConfig::setCaptureDelay);
 
-    mCaptureOnClick = new QCheckBox(this);
-    mCaptureOnClick->setText(i18n("On Click"));
+    mCaptureOnClick = new QCheckBox(i18n("On Click"), this);
     mCaptureOnClick->setToolTip(i18n("Wait for a mouse click before capturing the screenshot image"));
     connect(mCaptureOnClick, &QCheckBox::stateChanged, this, &KSWidget::onClickStateChanged);
-    connect(mCaptureOnClick, &QCheckBox::stateChanged, configManager, &SpectacleConfig::setOnClickChecked);
+    connect(mCaptureOnClick, &QCheckBox::clicked, configManager, &SpectacleConfig::setOnClickChecked);
 
     mDelayLayout = new QHBoxLayout;
     mDelayLayout->addWidget(mDelayMsec);
@@ -89,72 +85,68 @@ KSWidget::KSWidget(QWidget *parent) :
     mCaptureModeForm->addRow(i18n("Delay:"), mDelayLayout);
     mCaptureModeForm->setContentsMargins(24, 0, 0, 0);
 
-    // the content options (mouse pointer, window decorations)
+    // options (mouse pointer, window decorations, quit after saving or copying)
 
     mContentOptionsLabel = new QLabel(this);
-    mContentOptionsLabel->setText(i18n("<b>Content Options</b>"));
+    mContentOptionsLabel->setText(i18n("<b>Options</b>"));
 
-    mMousePointer = new QCheckBox(this);
-    mMousePointer->setText(i18n("Include mouse pointer"));
-    mMousePointer->setToolTip(i18n("Show the mouse cursor in the screeenshot image"));
-    connect(mMousePointer, &QCheckBox::stateChanged, configManager, &SpectacleConfig::setIncludePointerChecked);
+    mMousePointer = new QCheckBox(i18n("Include mouse pointer"), this);
+    mMousePointer->setToolTip(i18n("Show the mouse cursor in the screenshot image"));
+    connect(mMousePointer, &QCheckBox::clicked, configManager, &SpectacleConfig::setIncludePointerChecked);
 
-    mWindowDecorations = new QCheckBox(this);
-    mWindowDecorations->setText(i18n("Include window titlebar and borders"));
+    mWindowDecorations = new QCheckBox(i18n("Include window titlebar and borders"), this);
     mWindowDecorations->setToolTip(i18n("Show the window title bar, the minimize/maximize/close buttons, and the window border"));
     mWindowDecorations->setEnabled(false);
-    connect(mWindowDecorations, &QCheckBox::stateChanged, configManager, &SpectacleConfig::setIncludeDecorationsChecked);
+    connect(mWindowDecorations, &QCheckBox::clicked, configManager, &SpectacleConfig::setIncludeDecorationsChecked);
 
-    mCaptureTransientOnly = new QCheckBox(this);
-    mCaptureTransientOnly->setText(i18n("Capture the current pop-up only"));
-    mCaptureTransientOnly->setToolTip(i18n("Capture only the current pop-up window (like a menu, tooltip etc). "
-                                           "If this is not enabled, the pop-up is captured along with the parent window"));
+    mCaptureTransientOnly = new QCheckBox(i18n("Capture the current pop-up only"), this);
+    mCaptureTransientOnly->setToolTip(i18n("Capture only the current pop-up window (like a menu, tooltip etc).\n"
+                                           "If disabled, the pop-up is captured along with the parent window"));
     mCaptureTransientOnly->setEnabled(false);
-    connect(mCaptureTransientOnly, &QCheckBox::stateChanged, configManager, &SpectacleConfig::setCaptureTransientWindowOnlyChecked);
+    connect(mCaptureTransientOnly, &QCheckBox::clicked, configManager, &SpectacleConfig::setCaptureTransientWindowOnlyChecked);
+
+    mQuitAfterSaveOrCopy = new QCheckBox(i18n("Quit after Save or Copy"), this);
+    mQuitAfterSaveOrCopy->setToolTip(i18n("Quit Spectacle after saving or copying the image"));
+    connect(mQuitAfterSaveOrCopy, &QCheckBox::clicked, configManager, &SpectacleConfig::setQuitAfterSaveOrCopyChecked);
 
     mContentOptionsForm = new QVBoxLayout;
     mContentOptionsForm->addWidget(mMousePointer);
     mContentOptionsForm->addWidget(mWindowDecorations);
     mContentOptionsForm->addWidget(mCaptureTransientOnly);
-    mContentOptionsForm->setSpacing(16);
+    mContentOptionsForm->addWidget(mQuitAfterSaveOrCopy);
     mContentOptionsForm->setContentsMargins(24, 0, 0, 0);
 
     // the take a new screenshot button
 
-    mTakeScreenshotButton = new QPushButton(this);
-    mTakeScreenshotButton->setText(i18n("Take a New Screenshot"));
+    mTakeScreenshotButton = new QPushButton(i18n("Take a New Screenshot"), this);
     mTakeScreenshotButton->setIcon(QIcon::fromTheme(QStringLiteral("spectacle")));
     mTakeScreenshotButton->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
     mTakeScreenshotButton->setFocus();
     connect(mTakeScreenshotButton, &QPushButton::clicked, this, &KSWidget::newScreenshotClicked);
 
-    QShortcut *shortcut = new QShortcut(QKeySequence(QKeySequence::New), mTakeScreenshotButton);
-    auto clickFunc = [&]() {
+    QShortcut *takeScreenshotShortcut = new QShortcut(QKeySequence(QKeySequence::New), mTakeScreenshotButton);
+    connect(takeScreenshotShortcut, &QShortcut::activated, [this]() {
         mTakeScreenshotButton->animateClick(100);
-        QTimer::singleShot(100, mTakeScreenshotButton, &QPushButton::click);
-    };
-    connect(shortcut, &QShortcut::activated, clickFunc);
+    });
 
     // finally, finish up the layouts
 
     mRightLayout = new QVBoxLayout;
     mRightLayout->addStretch(1);
     mRightLayout->addWidget(mCaptureModeLabel);
-    mRightLayout->addSpacing(10);
     mRightLayout->addLayout(mCaptureModeForm);
     mRightLayout->addStretch(1);
     mRightLayout->addWidget(mContentOptionsLabel);
-    mRightLayout->addSpacing(10);
     mRightLayout->addLayout(mContentOptionsForm);
     mRightLayout->addStretch(10);
     mRightLayout->addWidget(mTakeScreenshotButton, 1, Qt::AlignHCenter);
-    mRightLayout->setContentsMargins(20, 0, 0, 10);
+    mRightLayout->setContentsMargins(10, 0, 0, 10);
 
     mMainLayout = new QGridLayout(this);
     mMainLayout->addWidget(mImageWidget, 0, 0, 1, 1);
     mMainLayout->addLayout(mRightLayout, 0, 1, 1, 1);
-    mMainLayout->setColumnMinimumWidth(0, 400);
-    mMainLayout->setColumnMinimumWidth(1, 400);
+    mMainLayout->setColumnMinimumWidth(0, 320);
+    mMainLayout->setColumnMinimumWidth(1, 320);
 
     // and read in the saved checkbox states and capture mode indices
 
@@ -162,12 +154,29 @@ KSWidget::KSWidget(QWidget *parent) :
     mWindowDecorations->setChecked    (configManager->includeDecorationsChecked());
     mCaptureOnClick->setChecked       (configManager->onClickChecked());
     mCaptureTransientOnly->setChecked (configManager->captureTransientWindowOnlyChecked());
+    mQuitAfterSaveOrCopy->setChecked  (configManager->quitAfterSaveOrCopyChecked());
     mCaptureArea->setCurrentIndex     (configManager->captureMode());
     mDelayMsec->setValue              (configManager->captureDelay());
 
     // done
 }
 
+int KSWidget::imagePaddingWidth() const
+{
+    int rightLayoutLeft = 0;
+    int rightLayoutRight = 0;
+    int mainLayoutRight = 0;
+
+    mRightLayout->getContentsMargins(&rightLayoutLeft, nullptr, &rightLayoutRight, nullptr);
+    mMainLayout->getContentsMargins(nullptr, nullptr, &mainLayoutRight, nullptr);
+
+    int paddingWidth = (rightLayoutLeft + rightLayoutRight + mainLayoutRight);
+    paddingWidth += mRightLayout->contentsRect().width();
+    paddingWidth += 2 * SpectacleImage::SHADOW_RADIUS; // image drop shadow
+
+    return paddingWidth;
+}
+
 // public slots
 
 void KSWidget::setScreenshotPixmap(const QPixmap &pixmap)
